#ifndef _CAST_PARAM_H_
#define _CAST_PARAM_H_

#include <ast/Param.h>

namespace cli {

class CastParam final : public Param {
	std::unique_ptr < cli::Arg > m_type;
	std::unique_ptr < Param > m_param;

public:
	CastParam ( std::unique_ptr < cli::Arg > type, std::unique_ptr < Param > param ) : m_type ( std::move ( type ) ), m_param ( std::move ( param ) ) {
	}

	virtual std::shared_ptr < abstraction::OperationAbstraction > translateAndEval ( const std::shared_ptr < abstraction::OperationAbstraction > & prev, Environment & environment ) const override {
		std::string type = m_type->eval ( environment );

		std::shared_ptr < abstraction::OperationAbstraction > translatedParam = m_param->translateAndEval ( prev, environment  );

		if ( abstraction::Registry::isCastNoOp ( type, translatedParam->getReturnType ( ) ) ) {
			return translatedParam;
		}

		bool normalize;

		std::shared_ptr < abstraction::OperationAbstraction > res = abstraction::Registry::getCastAbstraction ( type, translatedParam->getReturnType ( ), normalize );
		res->attachInput ( translatedParam, 0 );
		res->eval ( );

		if ( normalize ) {
			std::shared_ptr < abstraction::OperationAbstraction > normalized = abstraction::Registry::getNormalizeAbstraction ( res->getReturnType ( ) );
			normalized->attachInput ( res, 0 );
			normalized->eval ( );

			res = normalized;
		}

		return res;
	}

};

} /* namespace cli */

#endif /* _CAST_PARAM_H_ */
