#ifndef _CLI_CAST_STATEMENT_H_
#define _CLI_CAST_STATEMENT_H_

#include <ast/Statement.h>

namespace cli {

class CastStatement final : public Statement {
	std::unique_ptr < cli::Arg > m_type;
	std::shared_ptr < Statement > m_statement;

public:
	CastStatement ( std::unique_ptr < cli::Arg > type, std::shared_ptr < Statement > statement ) : m_type ( std::move ( type ) ), m_statement ( statement ) {
	}

	virtual std::shared_ptr < abstraction::OperationAbstraction > translateAndEval ( const std::shared_ptr < abstraction::OperationAbstraction > & prev, Environment & environment ) const override {
		std::string type = m_type->eval ( environment );

		std::shared_ptr < abstraction::OperationAbstraction > translatedStatement = m_statement->translateAndEval ( prev, environment );

		if ( abstraction::Registry::isCastNoOp ( type, translatedStatement->getReturnType ( ) ) ) {
			return translatedStatement;
		}

		bool normalize;

		std::shared_ptr < abstraction::OperationAbstraction > res = abstraction::Registry::getCastAbstraction ( type, translatedStatement->getReturnType ( ), normalize );
		res->attachInput ( translatedStatement, 0 );
		res->eval ( );

		if ( normalize ) {
			std::shared_ptr < abstraction::OperationAbstraction > normalized = abstraction::Registry::getNormalizeAbstraction ( res->getReturnType ( ) );
			normalized->attachInput ( res, 0 );
			normalized->eval ( );

			res = normalized;
		}

		return res;
	}

};

} /* namespace cli */

#endif /* _CLI_CAST_STATEMENT_H_ */
