/*
 * UnaryOperationAbstraction.hpp
 *
 *  Created on: 11. 7. 2017
 *	  Author: Jan Travnicek
 */

#ifndef _UNARY_OPERATION_ABSTRACTION_HPP_
#define _UNARY_OPERATION_ABSTRACTION_HPP_

#include <abstraction/ResultClassSelector.hpp>
#include <tuple>
#include <abstraction/Registry.h>

namespace abstraction {

template < class ReturnType, class ParamType >
class UnaryOperationAbstraction : public ResultClassSelector < ReturnType >::OutputClass {
protected:
	std::shared_ptr < ValueOperationAbstraction < typename std::decay < ParamType >::type > > m_param;

private:
	virtual bool attachInput ( const std::shared_ptr < OperationAbstraction > & input, unsigned index ) override {
		if ( index != 0 )
			throw exception::CommonException ( "Out of range index: " + ext::to_string ( index ) + " max: " + ext::to_string ( numberOfParams ( ) ) + "." );

		std::shared_ptr < ValueOperationAbstraction < ParamType > > validData = std::dynamic_pointer_cast < ValueOperationAbstraction < ParamType > > ( input );
		if ( validData ) {
			m_param = validData;
			return true;
		} else {
			return false;
		}
	}

	virtual bool detachInput ( const std::shared_ptr < OperationAbstraction > & input ) override {
		if ( m_param != input )
			return false;

		m_param = nullptr;
		return true;
	}

public:
	UnaryOperationAbstraction ( ) {
	}

	virtual bool inputsReady ( ) const override {
		return m_param && m_param->isReady ( );
	}

	virtual bool inputsAttached ( ) const override {
		return ( bool ) m_param;
	}

	virtual bool eval ( ) override {
		if ( ! inputsAttached ( ) )
			return false;

		if ( this->cached ( ) )
			return true;

		this->m_param->eval ( );

		return this->run ( );
	}

	virtual unsigned numberOfParams ( ) const override {
		return 1;
	}

	virtual ext::type_index getParamTypeIndex ( unsigned index ) const override {
		if ( index != 0 )
			throw std::out_of_range ( "Invalid input index" );

		return ext::type_index ( typeid ( ParamType ) );
	}

};

} /* namespace abstraction */

#endif /* _UNARY_OPERATION_ABSTRACTION_HPP_ */
